//
// CutListProcessor.cpp
//
//  Process a list of cuts
//
#include "single_top_analysis/CutListProcessor.hpp"

#include "single_top_analysis/myTH1F.hpp"
#include "TDirectory.h"

#include <sstream>
#include <iostream>
#include <iomanip>
#include <string>
#include <algorithm>
#include <functional>

bool CutListProcessor::_g_debug = false;
EventContainer *CutListProcessor::_currentRE = 0;
BaseCut *CutListProcessor::_currentPackage = 0;

std::vector<CutListProcessor::auto_insert*> &CutListProcessor::defaultCuts (void)
{
  static std::vector<CutListProcessor::auto_insert*> g_cuts;
  return g_cuts;
}

//
// CutListProcessor
//
//  Process a list of cuts, dude!
//
CutListProcessor::CutListProcessor (const std::string &name, bool always_pass)
  : _name (name), _always_pass (always_pass)
{
  std::for_each (defaultCuts().begin(), defaultCuts().end(),
		 std::bind2nd (std::mem_fun (&auto_insert::AddCuts), this));
}

//
// GetCutName
//
//  Return the name
//
std::string CutListProcessor::GetCutName (void)
{
  return _name;
}

//
// BookHistogram
//
//  Pass the booking on down! We don't have any of them ourselves for now!
//
void CutListProcessor::BookHistogram ()
{
  //
  // Let em know what happened. :-)
  //
  //  Watch for the case that we have no sub-cuts running. In this case
  // don't generate the who ran histo.
  //
  
  if (_cut_histos.size() > 0) {
    _whoran = new TH1F ("whoran", "How Many Times Did Each Base Cut Run",
			_cut_histos.size(), 0, _cut_histos.size());
    _whoran->SetXTitle ("Cut Number");
    
    //
    // Now, book all the guys below that....
    //
    
    int cutnum = 0;
    for(std::vector<BaseCut *>::iterator itask=_cut_histos.begin(); itask!=_cut_histos.end(); ++itask){
      std::ostringstream dirn;
      _whoran->GetXaxis()->SetBinLabel (cutnum+1, (*itask)->GetCutName().c_str());
      dirn << std::setw(2) << std::setfill('0') << cutnum << std::setw(0) << "_" << (*itask)->GetCutName();
      cutnum++;
      std::string dirName = dirn.str();
      std::cout << "     " <<  dirName << std::endl;
      TDirectory *directory = GetDirectory()->mkdir(dirName.c_str()," ");
      directory->cd();
      (*itask) -> SetDirectory(directory);
      (*itask) -> BookHistogram();
    }
  }
}

//
// PrintCut
//
//  Pass it on down!
//
void CutListProcessor::PrintCut (const std::string &indent)
{
  // call printing method at the end of the run. The PrintCut() method is 
  // usefully implemented in Selection class so far.
  std::cout << indent << "Results from Cut List '" << _name << "':" << std::endl;
  for(std::vector<BaseCut *>::iterator itask=_cut_histos.begin(); itask!=_cut_histos.end(); ++itask){
    (*itask)->PrintCut(indent + "  ");
  }
}

//
// Apply
//
//  Run all the cuts!
//
bool CutListProcessor::Apply (EventContainer *obj)
{
  BaseCut *oldBaseCut = _currentPackage;
  EventContainer *oldRE = _currentRE;
  try {
    _currentRE = obj;
    
    //loop on all the task added
    int count = 0;
    for(std::vector<BaseCut *>::iterator itask=_cut_histos.begin(); itask!=_cut_histos.end(); ++itask){
      // changing directory to fill histograms of tasks
      if (_g_debug) {
	std::cout << "Apply " << (*itask)->GetCutName() << std::endl;
      }
      (*itask)->GetDirectory()->cd();
      _whoran->Fill (count);
      count++;
      _currentPackage = *itask;
      if ( !(*itask) -> Apply() ) {
	clean_up_run (_cut_histos.begin(), itask+1);
	return _always_pass;
      }
      //
    }  // end loop  on task
    clean_up_run (_cut_histos.begin(), _cut_histos.end());
    return true;
  } catch (...) {
    _currentRE = oldRE;
    _currentPackage = oldBaseCut;
    throw;
  }
}

void CutListProcessor::clean_up_run (std::vector<BaseCut *>::iterator start,
				     std::vector<BaseCut *>::iterator end)
{
  while (start != end) {
    end--;
    (*end)->DoneWithProcessingLoop();
  }
}

//
// AddCut
//
//  Add a cut into the sequence
//
void CutListProcessor::AddCut (BaseCut *task)
{
  _cut_histos.push_back(task);
}


